from cms.extensions.toolbar import ExtensionToolbar
from cms.toolbar_pool import toolbar_pool
from django.utils.translation import ugettext_lazy as _

from .models import MetaTagPageExtension


@toolbar_pool.register
class MetaTagPageExtensionToolbar(ExtensionToolbar):
    model = MetaTagPageExtension

    def populate(self):
        current_page_menu = self._setup_extension_toolbar()

        if current_page_menu:
            page_extension, url = self.get_page_extension_admin()

            if url:
                current_page_menu.add_modal_item(_('Meta Tags'), url=url, disabled=not self.toolbar.edit_mode)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-11-08 08:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('meta_tagger', '0002_auto_20160527_1656'),
    ]

    operations = [
        migrations.AddField(
            model_name='metatagpageextension',
            name='robots_disallow',
            field=models.BooleanField(default=False,
                                      help_text="Adds this page with the 'Disallow' instruction to the robots.txt file. Search engines won't crawl it. Consider deactivating the indexing checkbox above to prevent indexing too.",
                                      verbose_name='Disallow'),
        ),
        migrations.AlterField(
            model_name='metatagpageextension',
            name='robots_following',
            field=models.BooleanField(default=True, help_text='Allows search engines to follow the links on this page.',
                                      verbose_name='Allow Following'),
        ),
        migrations.AlterField(
            model_name='metatagpageextension',
            name='robots_indexing',
            field=models.BooleanField(default=True,
                                      help_text='Allows search engines to include this page in search results.',
                                      verbose_name='Allow Indexing'),
        ),
    ]



from contextlib import suppress

from cms import __version__ as cms_version
from cms.models import Title
from django.contrib.sites.models import Site
from django.contrib.sites.shortcuts import get_current_site
from django.db.models import Q
from django.shortcuts import render
from django.urls import NoReverseMatch, reverse
from django.views import View
from pkg_resources import parse_version

from meta_tagger.models import MetaTagPageExtension


class RobotsView(View):
    def get(self, request, *args, **kwargs):
        context = {
            'pages': self.get_pages(),
            'sitemap_url': self.get_sitemap_url()
        }
        return render(request, 'meta_tagger/robots.txt', context, content_type='text/plain')

    @staticmethod
    def get_page_title_queryset():
        site = Site.objects.get_current()
        queryset = Title.objects.public().filter(Q(redirect='') | Q(redirect__isnull=True), page__login_required=False)
        if parse_version(cms_version) > parse_version('3.5'):
            return queryset.filter(page__node__site=site).order_by('page__node__path')
        else:
            return queryset.filter(page__site=site).order_by('page__path')

    def get_pages(self) -> list:
        pages = []
        for title in self.get_page_title_queryset():
            with suppress(MetaTagPageExtension.DoesNotExist):
                if title.page.metatagpageextension.robots_disallow:
                    pages.append({
                        'instruction': 'Disallow',
                        'path': title.page.get_absolute_url(title.language)
                    })
        return pages

    def get_sitemap_url(self) -> str:
        try:
            path = reverse('sitemap')
        except NoReverseMatch:
            return ''
        scheme = 'https' if self.request.is_secure() else 'http'
        site = get_current_site(self.request)
        return '{scheme}://{site}{path}'.format(scheme=scheme, site=site, path=path)



# -*- coding: utf-8 -*-
from cms.extensions import PageExtensionAdmin
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import MetaTagPageExtension


@admin.register(MetaTagPageExtension)
class MetaPageExtensionAdmin(PageExtensionAdmin):
    fieldsets = (
        (_('Search Engines'), {
            'fields': ('robots_indexing', 'robots_following', 'robots_disallow')
        }),
        (_('Open Graph'), {
            'fields': ('og_image', ('og_image_width', 'og_image_height'))
        })
    )
